package classes.utils
{
	import classes.elements.CreateLabels;
	import classes.elements.NavButton;
	import classes.elements.PreloaderObject;
	import classes.events.NavigationEvent;
	
	import flash.display.Bitmap;
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.TimerEvent;
	import flash.net.URLRequest;
	import flash.text.AntiAliasType;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.utils.Timer;

	public class SlideView extends Sprite
	{
		//----------------------------------------------------------------------
        //
        // Private Properties
        //
        //----------------------------------------------------------------------
        
        private var _container		:Sprite		= new Sprite();
        
        private var _slideOne		:Sprite		= new Sprite();
        
        private var _slideTwo		:Sprite		= new Sprite();
        
        private var _currentSlide	:Sprite		= new Sprite();
        
        private var _previousSlide	:Sprite		= new Sprite();
        
        private var _currentIndex	:int		= -1;
        
        private var _xmlProperties	:XML;
        
        private var _slideTimer		:Timer;
        
        private var _loader			:Loader;
        
        private var _preloader		:PreloaderObject;
        
        private var _tweenType		:String;
        
        private var _tweenEase		:Function;
        
        private var _tweenManager	:TweenManager;
        
        private var _getEasing		:GetEasing	= new GetEasing();
        
        private var _navDirection	:String		= "next_slide";
        
        //----------------------------------------------------------------------
        //
        // Constructor
        //
        //----------------------------------------------------------------------
        
		public function SlideView()
		{
			addEventListener( Event.ADDED_TO_STAGE, onStageHandler, false, 0, true );
		}
		
		//----------------------------------------------------------------------
        //
        // Private Methods
        //
        //----------------------------------------------------------------------
        
        private function onStageHandler( e:Event ):void
        {
        	removeEventListener( Event.ADDED_TO_STAGE, onStageHandler );
        	
        	_xmlProperties		= DynamicSlideShow.xmlProperties;
        	
        	_tweenType			= _xmlProperties.transitionType;
        	
        	_tweenEase			= _getEasing.getEasingType( _xmlProperties.transitionEasing );
        	
        	_slideTimer			= new Timer( _xmlProperties.delayTime );
        	_slideTimer.addEventListener( TimerEvent.TIMER, updateViewHandler, false, 0, true );
        	
        	addChild( _container );
        	
        	_container.addChild( _slideOne );
        	_container.addChild( _slideTwo );
        	
        	_currentSlide		= _slideTwo;
        	
        	_preloader			= new PreloaderObject();
        	_preloader.createLoadBar( _xmlProperties.preloaderColor, _xmlProperties.preloaderHeight );
        	
        	_tweenManager		= new TweenManager();
        	_tweenManager.addEventListener( Event.COMPLETE, startTimer, false, 0, true );
        	addChild( _tweenManager );
        	
        	if( DynamicSlideShow.displayLabels )
        	{
        		var createlabels:CreateLabels = new CreateLabels( _slideOne, _slideTwo );
        	}
        	
        	if( DynamicSlideShow.useNavigation )
        	{
        		var navNext:NavButton = new NavButton( _xmlProperties.navNextPath );
        		navNext.name = "navNext";
        		navNext.addEventListener( NavigationEvent.NEXT_SLIDE, updateViewNavHandler, false, 0, true );
        		navNext.x	= _xmlProperties.navNextXpos;
        		navNext.y	= _xmlProperties.navNextYpos;
        		addChild( navNext );
        		
        		var navPrev:NavButton = new NavButton( _xmlProperties.navPrevPath );
        		navPrev.name = "navPrev";
        		navPrev.addEventListener( NavigationEvent.PREV_SLIDE, updateViewNavHandler, false, 0, true );
        		navPrev.x	= _xmlProperties.navPrevXpos;
        		navPrev.y	= _xmlProperties.navPrevYpos;
        		addChild( navPrev );
        	}
        	
        	updateViewHandler( new TimerEvent( TimerEvent.TIMER ) );
        }
        
        private function updateViewHandler( e:TimerEvent ):void
        {
        	if( _slideTimer.running )
        	{
        		_slideTimer.stop();
        	}
        	
        	_currentIndex ++;
        	
        	if( _currentIndex == DynamicSlideShow.totalSlides )
        	{
        		_currentIndex = 0;
        	}
        	
        	switchAndLoad();
        }
        
        private function updateViewNavHandler( e:NavigationEvent ):void
        {
        	switch( e.type )
        	{
        		case "next_slide" :
        		_currentIndex ++
        		
        		if( _currentIndex == DynamicSlideShow.totalSlides )
        		{
        			_currentIndex = 0;
        		}
        		break;
        		
        		case "prev_slide" :
        		_currentIndex --
        		
        		if( _currentIndex == -1 )
        		{
        			_currentIndex = DynamicSlideShow.totalSlides - 1;
        		}
        		break;
        	}
        	
        	_navDirection = e.type;
        	
        	switchAndLoad();
        }
        
        private function switchAndLoad():void
        {
        	if( _currentSlide == _slideOne )
        	{
        		_currentSlide = _slideTwo;
        		_previousSlide = _slideOne;
        	}
        	else
        	{
        		_currentSlide = _slideOne;
        		_previousSlide = _slideTwo;
        	}
        	
        	_tweenManager.currSlide	= _currentSlide;
        	_tweenManager.prevSlide	= _previousSlide;
        	_tweenManager.tweenType	= _tweenType;
        	_tweenManager.tweenEase	= _tweenEase;
        	
        	_tweenManager.RestoreTweens( _navDirection );
        	
        	if( _currentSlide.numChildren > 1 )
        	{
        		_currentSlide.removeChildAt( 0 );
        	}
        	
        	_container.swapChildren( _slideOne, _slideTwo );
        	
        	_loader = new Loader();
        	_loader.contentLoaderInfo.addEventListener( Event.COMPLETE, transitionSlides, false, 0, true );
        	_loader.contentLoaderInfo.addEventListener( Event.OPEN, loadOpenHandler, false, 0, true );
        	_loader.contentLoaderInfo.addEventListener( ProgressEvent.PROGRESS, loadProgressHandler, false, 0, true );
        	_loader.contentLoaderInfo.addEventListener( IOErrorEvent.IO_ERROR, loadErrorHandler, false, 0, true );
        	
        	_loader.load( new URLRequest( DynamicSlideShow.imagesArray[ _currentIndex ] ) );
        }
        
        private function transitionSlides( e:Event ):void
        {
        	var bitmap:Bitmap 	= _loader.content as Bitmap;
        	bitmap.smoothing	= true;
        	
        	if( DynamicSlideShow.displayLabels )
        	{
        		var label_box:Sprite			= _currentSlide.getChildAt( 0 ) as Sprite;
        		var label_bg:Sprite				= label_box.getChildAt( 0 ) as Sprite;
        		var label_tf:TextField			= label_box.getChildAt( 1 ) as TextField;
        		
        		label_box.alpha					= 1;
        		
        		label_tf.width					= stage.stageWidth - label_tf.x;
        		label_tf.styleSheet				= FontHandler._cssStyleSheet;
        		label_tf.antiAliasType			= AntiAliasType.ADVANCED;
        		label_tf.autoSize				= TextFieldAutoSize.LEFT;
        		label_tf.multiline				= true;
        		label_tf.embedFonts				= true;
        		label_tf.htmlText				= DynamicSlideShow.labelsArray[ _currentIndex ];
        		
        		label_bg.height					= label_tf.height + Number( _xmlProperties.labelBoxTopPadding ) + Number( _xmlProperties.labelBoxBottomPadding );
        		
        		_currentSlide.addChildAt( bitmap, 0 );
        	}
        	else
        	{
        		_currentSlide.addChild( bitmap );
        	}
        	
        	_tweenManager.TweenObjects( _navDirection );
        }
        
        private function startTimer( e:Event ):void
        {
        	if( !DynamicSlideShow.useNavigation )
        	{
        		_slideTimer.start();
        	}
        }
        
        private function loadOpenHandler( e:Event ):void
        {
        	_loader.removeEventListener( Event.OPEN, loadOpenHandler );
        	
        	addChild( _preloader );
        }
        
        private function loadProgressHandler( e:ProgressEvent ):void
        {
        	var percent:Number = ( 1.0 / e.bytesTotal ) * e.bytesLoaded;
        	
        	if( percent == 1 )
        	{
        		_loader.removeEventListener( ProgressEvent.PROGRESS, loadProgressHandler );
        		
        		removeChild( _preloader );
        	}
        	else
        	{
        		_preloader.loadBar.width = stage.stageWidth * percent;
        	}
        }
		
		private function loadErrorHandler( e:IOErrorEvent ):void
		{
			// ignore errors
		}
	}
}